import requests
import json

class TsugiLaunch():
    """Holds the launch data for a Tsugi Launch
    """
    user = None
    context = None
    link = None
    result = None
    service = None
    complete = False
    valid = False
    message = None
    detail = None
    redirecturl = None
    ltirow = None
    lti_launch = None
    lti_launch_debug = None

    def __init__(self, request=None) :
        if request is not None :
            self.from_request(request)
        else : 
            emptyrow = dict()
            self.load(emptyrow)

    def from_request(self, request) :
        self.lti_launch = request.session.get('lti_launch')
        self.lti_launch_debug = json.dumps(self.lti_launch, indent=4)
        self.load(self.lti_launch.get('lti'))

    def load(self, ltirow) : 
        self.ltirow = dict(ltirow) # copy 
        self.key = TsugiKey(self)
        self.context = TsugiContext(self)
        self.user = TsugiUser(self)
        self.link = TsugiLink(self)
        self.service = TsugiService(self)
        self.result = TsugiResult(self, self.service)

    def lti_sha256(self, value) :
        if value is None : return value
        return hashlib.sha256(value).hexdigest()

class TsugiKey() :
    def __init__(self, launch) :
        self.launch = launch      # reference
        self.id = launch.ltirow.get('key_id')
        # self.title = launch.ltirow.get('key_title')
        # self.settings = launch.ltirow.get('key_settings')

class TsugiContext() :
    def __init__(self, launch) :
        self.launch = launch      # reference
        self.id = launch.ltirow.get('context_id')
        self.title = launch.ltirow.get('context_title')
        self.settings = launch.ltirow.get('context_settings')

class TsugiUser() :
    def __init__(self, launch) :
        self.launch = launch      # reference
        self.id = launch.ltirow.get('user_id')
        self.displayname = launch.ltirow.get('user_displayname')
        self.email = launch.ltirow.get('user_email')
        self.image = launch.ltirow.get('user_image')
        self.role = int(launch.ltirow.get('role',0))

    def instructor(self) : return self.role >= 1000
    def tenantAdmin(self) : return self.role >= 5000
    def rootAdmin(self) : return self.role >= 10000

class TsugiLink() :
    def __init__(self, launch) :
        self.launch = launch      # reference
        self.id = launch.ltirow.get('link_id')
        self.title = launch.ltirow.get('link_title')
        self.path = launch.ltirow.get('link_path')
        self.settings = launch.ltirow.get('link_settings')
        self.settings_url = launch.ltirow.get('link_settings_url')

class TsugiService() :
    def __init__(self, launch) :
        self.launch = launch      # reference
        self.id = launch.ltirow.get('link_id')
        self.url = launch.ltirow.get('service_key')

class TsugiResult() :
    def __init__(self, launch, service) :
        self.launch = launch      # reference
        self.id = launch.ltirow.get('link_id')
        self.service = service
        self.sourcedid = launch.ltirow.get('sourcedid')
        self.url = launch.ltirow.get('result_url')

    def gradeSend(self,grade,comment) :
        print('gradeSend', grade, comment)
        callback = self.launch.lti_launch.get('callback')
        endpoint = callback.get('endpoint')
        token = callback.get('token')

        rpc = { 'token' : token,
                'object' : 'result',
                'method' : 'gradeSend',
                'p1' : grade,
                'p2' : comment
        }
        print(endpoint, rpc)

        r = requests.post(endpoint, data = rpc)
        print(r.status_code)
        print(r.headers)
        print(r.text)

        return r.text






""" Constants for cookieless """
LINKS_RE = r'<a(?P<pre_href>[^>]*?)href=["\'](?P<in_href>[^"\']*?)(?P<anchor>#\S+)?["\'](?P<post_href>[^>]*?)>'

DEFAULT_SETTINGS = {
    "REWRITE": True,
    "USE_GET": True,
    "CLIENT_ID": True,
    "HOSTS": [],
    "NO_COOKIE_PERSIST": True,
    "URL_SPECIFIC": False,
    "DELETE_COOKIES": False,
}



from django.http import HttpResponse
from django.http import HttpResponseRedirect
from django.shortcuts import render, redirect
from django.urls import reverse_lazy

from django.conf import settings
from django.views import View

import requests, urllib
# from django_tsugi.LTIX import *

import jwt
from jwcrypto import jwk

TSUGI_JWK_LIST = dict()
TSUGI_ERROR_URL = "https://www.tsugi.org/djangoerror"
# TSUGI_ERROR_URL = "http://localhost:8888/tsugi-org/djangoerror"

class LaunchView(View) :

    def launcherror(self, error, detail=False) :
        url = TSUGI_ERROR_URL + '?detail=' + urllib.parse.urlencode({'detail': error})
        # retval = HttpResponseRedirect(url)
        retval = redirect(url)
        if ( detail ) : retval['X-Tsugi-Detail'] = detail
        return retval

    def get(self, request, success_url) :
        return self.launcherror('This is a Launch URL, expecting a POST with a JWT to initiate a launch')


    def post(self, request, success_url) :
        encoded = request.POST.get('JWT')
        if encoded is None:
            return self.launcherror('This URL is expecting a POST with a JWT to initiate a launch')

        try:
            header = jwt.get_unverified_header(encoded)
        except:
            return self.launcherror('Could not load header from JWT',encoded)

        try:
            kid = header.get('kid')
        except:
            return self.launcherror('The JWT is missing a key ID (kid)', encoded)

        # Check if we already have the kid
        public_key = TSUGI_JWK_LIST.get(kid, False)

        # Lets get the keyset, retrieve the key, and cache it
        if public_key == False :
            keyset_proxy = False
            try :
                if settings.TSUGI_PROXY is not False: keyset_proxy = {'http' : settings.TSUGI_PROXY}
            except:
                pass

            try :
                keyset_url = settings.TSUGI_KEYSET
            except :
                print("Please set TSUGI_KEYSET in your settings.py, using dev1.tsugicloud.org as default")
                keyset_url = "https://dev1.tsugicloud.org/tsugi/lti/keyset-ext"

            try:
                print(keyset_proxy)
                dat = requests.get(keyset_url, proxies=keyset_proxy).text
            except:
                dat = ""

            if len(dat) < 1 :
                return self.launcherror('Could not load keyset from '+keyset_url)

            # https://github.com/latchset/jwcrypto/blob/master/jwcrypto/tests.py
            # https://jwcrypto.readthedocs.io/en/latest/jwk.html#classes
            try:
                ks = jwk.JWKSet()
                ks.import_keyset(dat)
            except:
                return self.launcherror('Could not parse keyset from '+keyset_url,encoded)


            try:
                k1 = ks.get_key(kid)
                public_key = jwk.JWK.export_to_pem(k1)
            except:
                return self.launcherror('Could not extract kid='+kid+' from '+keyset_url,encoded)

            if len(TSUGI_JWK_LIST) > 10 :
                TSUGI_JWK_LIST.clear()
            TSUGI_JWK_LIST[kid] = public_key

        try:
            lti_launch = jwt.decode(encoded, public_key, algorithms=['RS256'])
            print('Forwarding valid launch')
            print(lti_launch)
        except:
            return self.launcherror('Could not validate JSON Web Token signature',encoded)

        request.session['lti_launch'] = lti_launch
        return redirect(reverse_lazy(success_url))

